#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

from .. import parts

class RipeRetailPart(parts.Part):

    def login(self, username, password):
        self.driver.get(self.login_url)

        form = self.driver.find_element(".form")
        username_input = form.find_element_by_name("username")
        self.driver.write_text(username_input, username)
        password_input = form.find_element_by_name("password")
        self.driver.write_text(password_input, password)
        self.driver.press_enter(password_input)

    def login_wait(self, username, password):
        self.login(username, password)
        self.waits.redirected_to(self.next_url)

    def select_size(self, size, gender = None, scale = None, open = True):
        """
        Opens the size selection window, selects the proper scale and size and
        applies that configuration by clicking 'Apply' and closing the window.

        Notice that if the "open" flag is unset the window is not opened.

        :type size: String
        :param size: The size to be picked.
        :type gender: String
        :param gender: The gender that is going to be picked.
        :type scale: String
        :param scale: The scale that is going to be picked.
        :type open: Boolean
        :param open: If the size modal window should be opened before selection.
        """

        if open: self.interactions.click_when_possible(".size:not(.disabled) .button-size")

        if gender:
            self.interactions.click_when_possible(
                ".size .button-gender",
                condition = lambda element: element.text == gender
            )

        if scale:
            self.interactions.click_when_possible(
                ".size .button-scale",
                condition = lambda element: element.text == str(scale)
            )

        self.interactions.click_when_possible(
            ".size .button-size",
            condition = lambda element: element.text == str(size)
        )

        self.interactions.click_when_possible(".size .button.button-primary.button-apply")
        self.waits.is_not_visible(".size .modal")

    def set_part(
        self,
        brand,
        model,
        part,
        material,
        color,
        part_text = None,
        material_text = None,
        color_text = None,
        verify = True
    ):
        """
        Makes a change to the customization of a part and checks that the pages
        mutates correctly, picking the right active parts, materials and colors,
        as well as properly switching the swatches.

        If the text parameters are passed an extra set of assertions are going
        to be performed to validate expected behaviour.

        :type brand: String
        :param brand: The brand of the model being customized.
        :type model: String
        :param model: The model being customized.
        :type part: String
        :param part: The technical name of the part being changed.
        :type material: String
        :param material: The technical name of the material to use for the part.
        :type color: String
        :param color: The technical name of the color to use for the part.
        :type part_text: String
        :param part_text: The expected label for the part after clicking.
        :type material_text: String
        :param material_text: The expected label for the material after clicking.
        :type color_text: String
        :param color_text: The expected label for the color after clicking.
        :type verify: bool
        :param verify: If a final assertion should be performed after the selection
        has been done (to verify the final status).
        """

        self.interactions.click_when_possible(
            ".pickers .button-part > p:not(.no-part)",
            condition = lambda e: e.text == part.upper()
        )
        self.interactions.click_when_possible(
            ".pickers .button-color[data-material='%s'][data-color='%s']" % (material, color)
        )

        if verify:
            self.assert_part(
                brand,
                model,
                part,
                material,
                color,
                part_text = part_text,
                material_text = material_text,
                color_text = color_text
            )

    def assert_part(
        self,
        brand,
        model,
        part,
        material,
        color,
        part_text = None,
        material_text = None,
        color_text = None
    ):
        """
        Checks that the part pickers have the expected state, meaning that the
        complete set of assertions are properly filled.

        If the text parameters are passed an extra set of assertions are going
        to be performed to validate expected behaviour.

        Notice that this assertion requires the changing of the current visual
        state, in the sense that the part tab is going to be switched to the
        one that is going to be asserted.

        :type brand: String
        :param brand: The brand of the model being customized.
        :type model: String
        :param model: The model being customized.
        :type part: String
        :param part: The technical name of the part being checked.
        :type material: String
        :param material: The technical name of the material used in the part.
        :type color: String
        :param color: The technical name of the color used in the part.
        :type part_text: String
        :param part_text: The expected label for the part.
        :type material_text: String
        :param material_text: The expected label for the material.
        :type color_text: String
        :param color_text: The expected label for the color.
        """

        self.interactions.click_when_possible(
            ".pickers .button-part",
            condition = lambda e: e.text == part.upper()
        )

        if part_text: self.waits.text(".button-part.active", part_text)
        if color_text: self.waits.text(".button-color.active", color_text)
        if material_text: self.waits.text(".button-material.active", material_text)

        self.waits.until(
            lambda d: self.core.assert_swatch(
                ".pickers .button-part.active .swatch > img",
                brand, model, material, color
            ),
            "Part swatch didn't have the expected image."
        )
        self.waits.until(
            lambda d: self.core.assert_swatch(
                ".pickers .button-color.active .swatch > img",
                brand, model, material, color
            ),
            "Color swatch didn't have the expected image."
        )

    @property
    def retail_url(self):
        base_url = appier.conf("BASE_URL", "https://ripe-retail-ci.platforme.com")
        base_url = appier.conf("RETAIL_URL", base_url)
        base_url = appier.conf("RIPE_RETAIL_URL", base_url)
        return base_url

    @property
    def username(self):
        username = appier.conf("RETAIL_USERNAME", "root")
        username = appier.conf("RIPE_RETAIL_USERNAME", username)
        return username

    @property
    def password(self):
        password = appier.conf("RETAIL_PASSWORD", "root")
        password = appier.conf("RIPE_RETAIL_PASSWORD", password)
        return password

    @property
    def base_url(self):
        return self.retail_url

    @property
    def api_url(self):
        return "%s/api" % self.retail_url

    @property
    def admin_url(self):
        return "%s/admin" % self.retail_url

    @property
    def admin_api_url(self):
        return "%s/api/admin" % self.retail_url

    @property
    def export_url(self):
        return "%s/export" % self.retail_url

    @property
    def export_api_url(self):
        return "%s/api/export" % self.retail_url

    @property
    def home_url(self):
        return "%s/" % self.retail_url

    @property
    def next_url(self):
        return self.home_url

    @property
    def login_url(self):
        return "%s/login" % self.retail_url

    @property
    def logout_url(self):
        return "%s/logout" % self.retail_url



#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

from .. import parts

class RipePulsePart(parts.Part):

    def login(self):
        self.id.login()

    def login_wait(self, redirect_url = None):
        self.id.login_wait(redirect_url = redirect_url)

    def logout(self):
        self.driver.get(self.signout_url)

    def logout_wait(self, redirect_url = None):
        redirect_url = redirect_url or self.home_url
        self.logout()
        self.waits.redirected_to(redirect_url)

    def click_order(self, number):
        self.interactions.click_when_possible(".table .id a[href^='/orders/%d']" % number)
        self.waits.redirected_to(self.order_url(number))
        self.waits.text(".title", "Order #%d" % number)

    def order_url(self, number):
        url = "%s/orders/%d" % (self.pulse_url, number)
        return url

    @property
    def pulse_url(self):
        base_url = appier.conf("BASE_URL", "https://ripe-pulse-ci.platforme.com")
        base_url = appier.conf("PULSE_URL", base_url)
        base_url = appier.conf("RIPE_PULSE_URL", base_url)
        return base_url

    @property
    def base_url(self):
        return self.pulse_url

    @property
    def home_url(self):
        return "%s/" % self.pulse_url

    @property
    def next_url(self):
        return self.orders_url

    @property
    def orders_url(self):
        return "%s/orders" % self.pulse_url

    @property
    def signout_url(self):
        return "%s/signout" % self.pulse_url



#!/usr/bin/python
# -*- coding: utf-8 -*-

import time

import appier

import logging

from . import errors

class TestCase(appier.Observable):

    def __init__(self, *args, **kwargs):
        appier.Observable.__init__(self, *args, **kwargs)
        self.runner = kwargs.get("runner", None)
        self.loader = kwargs.get("loader", None)
        self.logger = kwargs.get("logger", self.__class__._build_logger())
        self.breadcrumbs = kwargs.get(
            "breadcrumbs",
            self.__class__._build_logger(
                name = "ripe-rainbow-breadcrumbs"
            )
        )

    @classmethod
    def _build_logger(cls, name = "ripe-rainbow", level = "INFO"):
        if hasattr(TestCase, "_logger"): return TestCase._logger
        level = appier.conf("LEVEL", level)
        level = appier.conf("RAINBOW_LEVEL", level)
        level = logging.getLevelName(level.upper())
        logger = logging.getLogger(name)
        handler = logging.StreamHandler()
        logger.addHandler(handler)
        handler.setLevel(level)
        logger.setLevel(level)
        TestCase._logger = logger
        return logger

    def before(self):
        pass

    def after(self):
        pass

    def succeeded(self, test, ctx = None):
        pass

    def failed(self, test, exception, ctx = None):
        pass

    def skipped(self, test):
        pass

    def run_all(self):
        for test in self.tests():
            self.run_test(test)

    def run_test(self, test, ctx = None):
        self.before()
        try:
            test()
            self.succeeded(test, ctx = ctx)
        except errors.SkipError:
            self.skipped(test)
            raise
        except Exception as exception:
            self.failed(test, exception, ctx = ctx)
            raise
        finally:
            self.after()

    def error(self, message):
        if not self.logger: return
        self.logger.error(message)

    def warning(self, message):
        if not self.logger: return
        self.logger.warning(message)

    def info(self, message):
        if not self.logger: return
        self.logger.info(message)

    def debug(self, message):
        if not self.logger: return
        self.logger.debug(message)

    def log_stack(self, method = None):
        pass

    def pause(self, timeout = 86400):
        time.sleep(timeout)

    def skip(self, message = None, reason = None):
        raise errors.SkipError(
            message = message,
            reason = reason or message
        )

    @property
    def tests(self):
        return [
            getattr(self, name) for name in dir(self)
            if not name == "tests" and hasattr(getattr(self, name), "test")
        ]



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os

import appier
import appier_admin
import appier_export

from .. import parts

class ProvisionPart(parts.Part):

    def reset(self, api = None, base = None, ctx = None):
        api = api or self.export_api(base = base, ctx = ctx)
        api.reset_database()

    def ripe_core(self, names = None, base_url = None, reset = True):
        names = names or (
            "account.json",
            "availability_rule.json",
            "ddp_rules.json",
            "exchange_rates.json",
            "extra_duty_rules.json",
            "factory_rule.json",
            "hs_code_maps.json",
            "hs_code_rules.json",
            "letter_rule.json",
            "order_state.json",
            "order.json",
            "price_rule.json",
            "product.json",
            "shipping_rules.json"
        )

        base_url = base_url or "https://cdn.platforme.com/data/ripe_core/%s"

        api = self.export_api(
            base = self.core,
            ctx = "core"
        )

        if reset: self.reset(api = api)

        for name in names:
            model = os.path.splitext(name)[0]
            items = appier.get(base_url % name)
            data = dict(items = items)
            api.import_model(model, data)

    def ripe_retail(self, names = None, base_url = None, reset = True):
        names = names or (
            "account.json",
            "brand.json",
            "store.json",
            "retail_account.json"
        )

        base_url = base_url or "https://cdn.platforme.com/data/ripe_retail/%s"

        api = self.export_api(
            base = self.retail,
            ctx = "retail"
        )

        if reset: self.reset(api = api)

        for name in names:
            model = os.path.splitext(name)[0]
            items = appier.get(base_url % name)
            data = dict(items = items)
            api.import_model(model, data)

    def ripe_retail_dummy(self):
        self._ripe_retail_extra("dummy")

    def ripe_retail_sergio_rossi(self):
        self._ripe_retail_extra("sergio_rossi")

    def ripe_retail_emilio_pucci(self):
        self._ripe_retail_extra("emilio_pucci")

    @property
    def api(self):
        return self.export_api()

    def export_api(self, base = None, ctx = None):
        base = base or self.core
        ctx = ctx or "core"
        name = "_export_api_%s" % ctx
        if hasattr(self, name) and getattr(self, name):
            return getattr(self, name)
        export_api = appier_export.API(
            base_url = base.export_api_url + "/",
            admin_url = base.admin_api_url + "/",
            username = base.username,
            password = base.password
        )
        setattr(self, name, export_api)
        return export_api

    def admin_api(self, base = None, ctx = None):
        base = base or self.core
        ctx = ctx or "core"
        name = "_admin_api_%s" % ctx
        if hasattr(self, name) and getattr(self, name):
            return getattr(self, name)
        admin_api = appier_admin.API(
            base_url = base.admin_api_url + "/",
            username = base.username,
            password = base.password
        )
        setattr(self, name, admin_api)
        return admin_api

    def _ripe_retail_extra(self, brand):
        url_prefix = "https://cdn.platforme.com/data/ripe_retail_%s/" % brand
        self.ripe_retail(
            names = (
                "brand.json",
                "store.json",
                "retail_account.json"
            ),
            base_url = url_prefix + "%s",
            reset = False
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

import re
import sys
import time

import appier
import appier_console

from . import util
from . import info
from . import errors
from . import loaders
from . import results
from . import interactive

class Runner(object):

    def __init__(self, loader = None, filter = None):
        self.loader = loader or self.loader_default
        self.filter = appier.conf("FILTER", filter)
        self.repeat = appier.conf("REPEAT", 1, cast = int)
        self.on_finish = []

        # creates the regular expression object that is going to
        # be used for test (method) name matching at runtime
        if self.filter: self.regex = re.compile("^.*%s.*$" % self.filter)
        else: self.regex = None

    def run(self):
        raise appier.NotImplementedError()

    def add_on_finish(self, method, safe = True):
        if safe and method in self.on_finish: return
        self.on_finish.append(method)

    def remove_on_finish(self, method, safe = True):
        if safe and not method in self.on_finish: return
        self.on_finish.remove(method)

    def run_on_finish(self):
        for method in self.on_finish:
            method()

    @property
    def test_suite(self):
        return self.loader.test_suite(runner = self)

    @property
    def loader_default(self):
        return None

class ConsoleRunner(Runner):

    def run(self):
        # sets the default result flag value to valid as the execution
        # is considered to be valid by default
        start_g = time.time()
        result = True
        passed = 0
        skipped = 0
        failed = 0
        passes = []
        skips = []
        failures = []

        # prints the header information on the product to be used to indicate
        # the proper execution of then console
        print(info.LABEL)
        print(interactive.InteractiveDriver.label_g())
        print("")

        try:
            # iterates over the complete set of test cases for the test
            # suite that has been provided by the current loader to run
            # their respective tests, under the current runner
            for test_case in self.test_suite:

                # sets the printed (overhead) flag as false indicating that
                # the name of the class is still pending printing
                printed = False

                # iterates over the complete set of test (methods) in the
                # test case to run then (if they are required)
                for test in test_case.tests:

                    # iterates over the amount of time the test should be run
                    # this is relevant for stress testing based execution
                    for index in appier.legacy.xrange(self.repeat):

                        # "gathers" the complete name for the test and verifies
                        # that it represents a valid one, against the regex
                        test_name = util.test_fullname(test)
                        if self.regex and not self.regex.match(test_name):
                            continue

                        # in case the name of the test case class is not printed
                        # runs the printing operation and unsets the flag
                        if not printed:
                            print("    %s" % util.test_case_fullname(test_case))
                            printed = True

                        # stores the starting time for the execution so that it's
                        # latter possible to calculate the duration of the test
                        start = time.time()

                        # builds the string prefix for the test to be used in the
                        # printing of the label for test execution
                        test_prefix = "[%d] " % (index + 1) if self.repeat > 1 else ""

                        try:
                            # starts the appier console context loader so that an animation
                            # is executed while the test is running (where it's possible)
                            with appier_console.ctx_loader(
                                spinner = "dots3",
                                template = "        %s%s {{spinner}}" % (test_prefix, test_name),
                                single = True,
                                eol = ""
                            ):
                                # executes the concrete logic of the test, making sure
                                # that an overall spinner is making the UI interactive
                                # notice that an extra context field is sent containing
                                # some of the details of the current execution session
                                test_case.run_test(
                                    test,
                                    ctx = dict(
                                        index = index,
                                        repeat = self.repeat
                                    )
                                )

                            test_name_s = appier_console.colored(test_name, color = appier_console.COLOR_CYAN)
                            mark_s = appier_console.colored("√", color = appier_console.COLOR_GREEN)
                            extra_s = ""
                            success = results.Result.build_success(test)
                            passes.append(success)
                            passed += 1
                        except errors.SkipError as exception:
                            test_name_s = appier_console.colored(test_name, color = appier_console.COLOR_CYAN)
                            mark_s = appier_console.colored("~", color = appier_console.COLOR_CYAN)
                            extra_s = " (" + exception.reason + ")" if exception.reason else ""
                            skip = results.Result.build_skip(test)
                            skips.append(skip)
                            skipped += 1
                        except appier.AssertionError as exception:
                            result = False
                            test_name_s = appier_console.colored(test_name, color = appier_console.COLOR_RED)
                            mark_s = appier_console.colored("✗", color = appier_console.COLOR_RED)
                            if test.bugs: extra_s = " (" + ", ".join(bug["url"] for bug in test.bugs if "url" in bug) + ")"
                            else: extra_s = ""
                            failure = results.Result.build_failure(test, exception)
                            failures.append(failure)
                            failed += 1
                        except Exception as exception:
                            result = False
                            failure = results.Result.build_failure(test, exception)
                            failures.append(failure)
                            test_name_s = appier_console.colored(test_name, color = appier_console.COLOR_RED)
                            mark_s = appier_console.colored("✗", color = appier_console.COLOR_RED)
                            if test.bugs: extra_s = " (" + ", ".join(bug["url"] for bug in test.bugs if "url" in bug) + ")"
                            else: extra_s = ""
                            failed += 1

                        # determines the kind of environment we're running on and taking that
                        # into account prints the proper output, standard is tty environment
                        # meaning that proper interaction is allowed
                        if appier_console.is_tty():
                            print("        %s %s%s%s" % (test_name_s, mark_s, extra_s, self._duration(start)))

                        # otherwise we're in a textual environment and only the extra part of
                        # the line is expected to be printed (not possible to go back in the line)
                        else:
                            print("%s%s%s" % (mark_s, extra_s, self._duration(start)))

                        # flushes the stdout and stderr so that the pending
                        # messages are properly handled by the output channels
                        sys.stdout.flush()
                        sys.stderr.flush()

                if printed: print("")
        finally:
            self.run_on_finish()

        passed_s = appier_console.colored(
            "%d passing" % passed,
            color = appier_console.COLOR_GREEN
        ) if passed else ""
        skipped_s = appier_console.colored(
            "%d skipped" % skipped,
            color = appier_console.COLOR_CYAN
        ) if skipped else ""
        failed_s = appier_console.colored(
            "%d failing" % failed,
            color = appier_console.COLOR_RED
        ) if failed else ""
        duration_s = self._duration(start_g, info = 0.0, warning = 3600.0, error = 14400.0)
        print("  %s%s%s%s%s" % (
                " " + passed_s if passed_s else "",
                " " + skipped_s if skipped_s else "",
                " " + failed_s if failed_s else "",
                "No tests executed" + failed_s if not passed_s and not skipped_s and not failed_s else "",
                duration_s
            )
        )

        # iterates over the complete set of failures to print
        # their description for better understanding of the
        # the underlying issues
        if failures: print("")
        for failure in failures: failure.print_result()

        print("")

        if result:
            print("The sky is blue and the sun is shining ☀️")
        else:
            print("There are some clouds in the sky 🌧️")

        return result

    def _duration(self, start, info = 5.0, warning = 5.0, error = 10.0):
        duration = time.time() - start
        if duration < info: return ""
        if duration >= error: color = appier_console.COLOR_RED
        elif duration >= warning: color = appier_console.COLOR_YELLOW
        else: color = appier_console.COLOR_CYAN
        return appier_console.colored(" (%.02fs)" % duration, color = color)

    @property
    def loader_default(self):
        return loaders.PathLoader(".")



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import traceback

import appier

from . import drivers

from .. import util
from .. import test_cases

class InteractiveTestCase(test_cases.TestCase):

    def __init__(self, *args, **kwargs):
        test_cases.TestCase.__init__(self, *args, **kwargs)
        self.options = kwargs
        self.driver = None
        self.timeout = appier.conf("TIMEOUT", 15, cast = int)
        self.timeout = self.options.get("timeout", self.timeout)

    def before(self):
        test_cases.TestCase.before(self)
        self.driver = self.load_driver()

    def after(self):
        if self.driver:
            self.driver.stop()
            self.driver = None
        test_cases.TestCase.after(self)

    def failed(self, test, exception, ctx = None):
        test_cases.TestCase.failed(self, test, exception)
        self._stacktrace(test, ctx = ctx)
        self._screenshot(test, ctx = ctx)

    def load_driver(self, start = True):
        driver_s = appier.conf("DRIVER", "selenium")
        driver_s = self.options.get("driver", driver_s)
        driver = drivers.InteractiveDriver.driver_g(driver_s)(self)
        if start: driver.start()
        return driver

    def _stacktrace(self, test, ctx = None):
        if not appier.conf("STACKTRACES", False, cast = bool): return
        if not self.driver: return
        ctx = ctx or {}
        index = ctx.get("index", 0)
        repeat = ctx.get("repeat", 1)
        extra_s = "-%d" % (index + 1) if repeat > 1 else ""
        base_path = appier.conf("STACKTRACES_PATH", ".")
        lines = traceback.format_exc().splitlines()
        lines = [line.decode("utf-8", "ignore") if appier.legacy.is_bytes(line) else\
            line for line in lines]
        test_name = util.test_fullname(test)
        if not os.path.exists(base_path): os.makedirs(base_path)
        stack_path = os.path.join(base_path, test_name + extra_s + ".log")
        stack_path = os.path.abspath(stack_path)
        stack_path = os.path.normpath(stack_path)
        stack_file = open(stack_path, "wb")
        try:
            for line in lines:
                stack_file.write(line.encode("utf-8"))
                stack_file.write(b"\n")
        finally:
            stack_file.close()

    def _screenshot(self, test, ctx = None):
        if not appier.conf("SCREENSHOTS", False, cast = bool): return
        if not self.driver: return
        ctx = ctx or {}
        index = ctx.get("index", 0)
        repeat = ctx.get("repeat", 1)
        extra_s = "-%d" % (index + 1) if repeat > 1 else ""
        base_path = appier.conf("SCREENSHOTS_PATH", ".")
        test_name = util.test_fullname(test)
        if not os.path.exists(base_path): os.makedirs(base_path)
        screen_path = os.path.join(base_path, test_name + extra_s + ".png")
        screen_path = os.path.abspath(screen_path)
        screen_path = os.path.normpath(screen_path)
        self.driver.screenshot(screen_path)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

from .. import parts

class RipeCorePart(parts.Part):

    def assert_swatch(self, selector, brand, model, material, color):
        """
        Checks that the img element identified by the selector points to the
        correct swatch. The correctness verification is performed by checking
        the `src` attribute of the element.

        This kind of assertion is critical to ensure proper responsiveness of
        the UI in accordance with part selection.

        :type selector: String
        :param selector: The selector for the img.
        :type brand: String
        :param brand: The brand of the swatch.
        :type model: String
        :param model: The model of the swatch.
        :type material: String
        :param material: The material the swatch should represent.
        :type color: String
        :param color: The color being shown in the shown.
        :rtype: bool
        :return: If the assertion was successful or not (propagation).
        """

        element = self.waits.element(selector)
        return self.assertions.match_url(
            element.get_attribute("src"),
            self.swatch_url,
            params = dict(
                brand = brand,
                model = model,
                material = material,
                color = color
            )
        )

    def wait_initials_image(self, selector, model, initials, profile = None):
        return self.waits.until(
            lambda d: self.assert_initials_image(
                selector,
                model,
                initials,
                profile = profile
            ),
            "Personalization image was not the expected one."
        )

    def assert_initials_image(self, selector, model, initials, profile = None):
        expected_params = dict(
            initials = initials,
            model = model
        )

        if profile: expected_params["initials_profile"] = profile

        element = self.waits.element(selector)
        src = element.get_attribute("src")

        return self.assertions.match_url(src, self.compose_url, params = expected_params)

    def order_url(self, number):
        return "%s/orders/%d" % (self.api_url, number)

    def report_pdf_url(self, number, key = None):
        url = "%s/orders/%d/report.pdf" % (self.api_url, number)
        if key: url += "?key=%s" % appier.util.quote(key)
        return url

    @property
    def core_url(self):
        core_url = appier.conf("CORE_URL", "https://ripe-core-ci.platforme.com")
        core_url = appier.conf("RIPE_CORE_URL", core_url)
        return core_url

    @property
    def username(self):
        username = appier.conf("CORE_USERNAME", "root")
        username = appier.conf("RIPE_CORE_USERNAME", username)
        return username

    @property
    def password(self):
        password = appier.conf("CORE_PASSWORD", "root")
        password = appier.conf("RIPE_CORE_PASSWORD", password)
        return password

    @property
    def base_url(self):
        return self.core_url

    @property
    def api_url(self):
        return "%s/api" % self.core_url

    @property
    def admin_url(self):
        return "%s/admin" % self.core_url

    @property
    def admin_api_url(self):
        return "%s/api/admin" % self.core_url

    @property
    def export_url(self):
        return "%s/export" % self.core_url

    @property
    def export_api_url(self):
        return "%s/api/export" % self.core_url

    @property
    def orders_url(self):
        return "%s/orders" % self.api_url

    @property
    def swatch_url(self):
        return "%s/swatch" % self.api_url

    @property
    def compose_url(self):
        return "%s/compose" % self.api_url



#!/usr/bin/python
# -*- coding: utf-8 -*-

import functools

def test(
    description = None,
    script_url = None,
    bug = None,
    bugs = []
):

    # in case the bugs are not defined tries to fallback
    # to the (single) bug parameter (convenience) and then
    # ensures that the bugs are set as a sequence
    if not bugs: bugs = [bug] if bug else bugs
    if not isinstance(bugs, (list, tuple)): bugs = [bugs]

    def decorator(function):

        function.test = True
        function.description = description or function.__name__
        function.script_url = script_url
        function.bugs = bugs

        @functools.wraps(function)
        def interceptor(*args, **kwargs):
            return function(*args, **kwargs)

        return interceptor

    return decorator



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import time

import appier

from .. import errors

class InteractiveDriver(object):

    def __init__(self, owner):
        self.owner = owner

    @staticmethod
    def driver_g(name):
        return globals()[name.capitalize() + "Driver"]

    @staticmethod
    def label_g():
        driver = appier.conf("DRIVER", "selenium")
        return InteractiveDriver.driver_g(driver).label()

    @classmethod
    def label(cls):
        return "Interactive"

    def start(self):
        pass

    def stop(self):
        pass

    def get_key(self, name):
        raise appier.NotImplementedError()

    def get(self, url):
        raise appier.NotImplementedError()

    def find_element(self, selector):
        raise appier.NotImplementedError()

    def find_elements(self, selector):
        raise appier.NotImplementedError()

    def find_by_name(self, name):
        raise appier.NotImplementedError()

    def focus(self, element):
        raise appier.NotImplementedError()

    def press_key(self, element, key):
        raise appier.NotImplementedError()

    def write_text(self, element, text):
        raise appier.NotImplementedError()

    def press_enter(self, element):
        return self.press_key(element, "enter")

    def click(self, element, ensure = True):
        raise appier.NotImplementedError()

    def ensure_visible(self, element, timeout = None):
        raise appier.NotImplementedError()

    def scroll_to(self, element, sleep = None):
        """
        Scrolls the element on which it's called into the visible area
        of the browser window.

        This operation should be performed in a deterministic way.

        :type element: Element
        :param element: The element to scroll into view.
        :type sleep: int
        :param sleep: The number of seconds to sleep after the scroll
        operation, important for smooth scrolls.
        """

        raise appier.NotImplementedError()

    def screenshot(self, file_path):
        raise appier.NotImplementedError()

    def wrap_outer(self, method, *args, **kwargs):
        return method(*args, **kwargs)

    def wrap_inner(self, method, *args, **kwargs):
        return method(*args, **kwargs)

    def safe(self, method, *args, **kwargs):
        return self.wrap_outer(self.wrap_inner, method, *args, **kwargs)

    @property
    def current_url(self):
        raise appier.NotImplementedError()

    def _wait(self, timeout = None):
        raise appier.NotImplementedError()

class SeleniumDriver(InteractiveDriver):

    def __init__(self, owner):
        InteractiveDriver.__init__(self, owner)
        self.secure = appier.conf("SEL_SECURE", False, cast = bool)
        self.browser = appier.conf("SEL_BROWSER", "chrome")
        self.maximized = appier.conf("SEL_MAXIMIZED", False, cast = bool)
        self.headless = appier.conf("SEL_HEADLESS", False, cast = bool)
        self.window_size = appier.conf("SEL_WINDOW_SIZE", "1920x1080")
        self.poll_frequency = appier.conf("SEL_POLL_FREQUENCY", None, cast = float)
        self.service_args = appier.conf("SEL_SERVICE_ARGS", [], cast = list)

    @classmethod
    def label(cls):
        import selenium.webdriver
        browser = cls.browser()
        return "Selenium %s (%s/%s)" % (
            selenium.webdriver.__version__,
            browser["name"],
            browser["version"]
        )

    @classmethod
    def browser(clas):
        import selenium.webdriver
        browser = appier.conf("SEL_BROWSER", "chrome")
        if browser == "chrome":
            options = selenium.webdriver.ChromeOptions()
            options.add_argument("--disable-extensions")
            options.add_argument("--headless")
            if sys.platform.startswith("linux"):
                options.add_argument("--disable-dev-shm-usage")
                options.add_argument("--no-sandbox")
            instance = selenium.webdriver.Chrome(options = options)
            try:
                name = instance.capabilities.get("browserName", None)
                version = instance.capabilities.get("browserVersion", None)
            finally:
                instance.quit()
        elif browser == "firefox":
            options = selenium.webdriver.FirefoxOptions()
            options.headless = True
            instance = selenium.webdriver.Firefox(options = options)
            try:
                name = instance.capabilities.get("browserName", None)
                version = instance.capabilities.get("browserVersion", None)
            finally:
                instance.quit()
        else:
            raise appier.OperationalError(
                message = "Unknown browser '%s'" % browser
            )
        return dict(
            name = name,
            version = version
        )

    def stop(self):
        self._flush_log()
        InteractiveDriver.stop(self)

    def get(self, url):
        return self.instance.get(url)

    def find_element(self, selector):
        return self.instance.find_element_by_css_selector(selector)

    def find_elements(self, selector):
        return self.instance.find_elements_by_css_selector(selector)

    def find_by_name(self, name):
        return self.instance.find_element_by_name(name)

    def find_element_by_css_selector(self, selector):
        return self.find_element(selector)

    def find_elements_by_css_selector(self, selector):
        return self.find_elements(selector)

    def find_element_by_name(self, name):
        return self.find_by_name(name)

    def focus(self, element):
        from selenium.webdriver.common.action_chains import ActionChains
        actions = ActionChains(self.instance)
        actions.move_to_element(element)
        actions.perform()

    def press_key(self, element, key):
        key = self._resolve_key(key)
        element.send_keys(key)
        return element

    def write_text(self, element, text):
        element.send_keys(text)
        return element

    def click(self, element, ensure = True):
        from selenium.common.exceptions import ElementClickInterceptedException, ElementNotVisibleException, WebDriverException

        try:
            # in case the ensure flag is set makes sure that the element
            # is visible (in an interactable way) to be then clicked
            if ensure: self.ensure_visible(element)

            # moves the mouse to the element according to the pre-defined
            # pivot, this will allow proper clicking
            self._move_to(element)

            # runs the click operation directly on the element without any
            # kind of previous interaction as expected
            element.click()
        except (
            ElementClickInterceptedException,
            ElementNotVisibleException,
            WebDriverException
        ) as exception:
            self.owner.breadcrumbs.debug("Element is not \"clickable\" because: %s (%s)" % (
                exception,
                exception.__class__
            ))
            return None

        # returns the element object to the caller so that it can
        # be piped in a chain of operations
        return element

    def ensure_visible(self, element, timeout = None):
        # starts the operation by moving the cursor to the outside of the element
        # this ensures that the cursor is not "moving over" the element
        self._move_outside(element, raise_e = False)

        # sets the initial value of the "entered" global variable and then registers the mouse
        # over event listener that will change the entered flag value
        self.instance.execute_script("window._entered = false")
        self.instance.execute_script("window._handler = function() { window._entered = true; };")
        self.instance.execute_script("arguments[0].addEventListener(\"mouseenter\", window._handler, true);", element)
        self.instance.execute_script("arguments[0].addEventListener(\"mouseover\", window._handler, true);", element)

        try:
            self._wait(timeout = timeout).until(
                lambda d: self._try_visible(element),
                "Element never became visible"
            )
        finally:
            self.instance.execute_script("delete window._entered")
            self.instance.execute_script("delete window._handler")
            self.instance.execute_script(
                "arguments[0].removeEventListener(\"mouseenter\", window._handler);",
                element
            )
            self.instance.execute_script(
                "arguments[0].removeEventListener(\"mouseover\", window._handler);",
                element
            )

        # returns the element object so that in can be chained in multiple
        # operations (functional design decision)
        return element

    def scroll_to(self, element, position = "center", sleep = None):
        # builds the proper options string taking into account the offset
        # in terms of positioning of the element in the scroll
        if position == "center": options = "{ block: \"center\", inline: \"center\" }"
        else: options = ""

        # as Selenium doesn't automatically support automatically scrolling
        # in elements inside the page, such as when using an iframe or
        # overflow scroll, therefore we must rely on Web API `Element.scrollIntoView()`
        # that allows proper scroll operation into element
        self.instance.execute_script("arguments[0].scrollIntoView(%s);" % options, element)

        # when triggering a smooth scroll, the element may take some time to
        # be displayed in the desired position, hence the optional sleep
        if sleep: time.sleep(sleep)

    def screenshot(self, file_path):
        self.instance.save_screenshot(file_path)

    def wrap_outer(self, method, *args, **kwargs):
        from selenium.common.exceptions import TimeoutException
        try:
            return method(*args, **kwargs)
        except TimeoutException as exception:
            raise errors.TimeoutError(message = exception.msg)

    def wrap_inner(self, method, *args, **kwargs):
        """
        Wraps the method being waited on to tolerate some exceptions since
        in most cases these are transitory conditions that shouldn't break
        the test.

        :type method: Function
        :param method: The method that is going to be executed in a wrapped
        fashion to properly handle exceptions.
        :rtype Function
        :return: The method wrapped on a try-catch for exceptions.
        """

        from selenium.common.exceptions import WebDriverException
        try: return method(*args, **kwargs)
        except (WebDriverException, AssertionError) as exception:
            self.owner.breadcrumbs.debug("Got exception while waiting: %s" % exception)
            return None

    @property
    def current_url(self):
        return self.instance.current_url

    @property
    def instance(self):
        cls = self.__class__
        if hasattr(cls, "_instance") and cls._instance:
            return cls._instance

        import selenium.webdriver

        if self.browser == "chrome":
            # creates the underlying instance of the Chrome driver
            # that is going to be used in the concrete execution
            cls._instance = selenium.webdriver.Chrome(
                options = self._selenium_options(self.browser),
                desired_capabilities = self._selenium_capabilities(self.browser),
                service_args = self.service_args or None
            )
        elif self.browser == "firefox":
            # creates the underlying Firefox instance using the
            # pre-defined options as expected
            cls._instance = selenium.webdriver.Firefox(
                options = self._selenium_options(self.browser),
                desired_capabilities = self._selenium_capabilities(self.browser),
                firefox_profile = self._selenium_profile(self.browser),
                service_args = self.service_args or None
            )
        else:
            raise appier.OperationalError(
                message = "Unknown browser '%s'" % self.brownser
            )

        # in case the browser should be started maximized, then
        # a new argument is added to the list of options
        if self.maximized:
            cls._instance.fullscreen_window()

        # in case the browser should have a specific window size
        # then splits the window size around the specific values
        if self.window_size:
            width, height = (int(value) for value in self.window_size.split("x"))
            cls._instance.set_window_size(width, height)

        # registers the destroy instance method to be called once
        # the runner is finished (proper cleanup)
        self.owner.runner.add_on_finish(self._destroy_instance)

        # returns the final instance of the driver to the caller
        # so that it can be used for operation
        return cls._instance

    def _destroy_instance(self):
        cls = self.__class__
        if not hasattr(cls, "_instance") or not self._instance:
            return
        cls._instance.quit()
        cls._instance = None

    def _selenium_options(self, browser):
        return getattr(self, "_selenium_options_%s" % browser)()

    def _selenium_options_chrome(self):
        import selenium.webdriver

        # crates the base object for the options to be used by
        # the Google Chrome browser
        options = selenium.webdriver.ChromeOptions()

        # adds some of the default arguments to be used for the
        # execution of the Google Chrome instance
        options.add_argument("--disable-extensions")

        # in case the headless mode was selected then an
        # extra argument is added to the set of options
        if self.headless:
            options.add_argument("--headless")

        # fixes issue when running within a docker container
        # since it typically uses /dev/shm shared memory with
        # little space making chrome crash when rendering large pages
        if sys.platform.startswith("linux"):
            options.add_argument("--disable-dev-shm-usage")
            options.add_argument("--no-sandbox")

        # returns the options to the calling method as expected
        # by the current infrastructure
        return options

    def _selenium_options_firefox(self):
        import selenium.webdriver

        # crates the base object for the options to be used by
        # the Mozilla Firefox browser
        options = selenium.webdriver.FirefoxOptions()

        # in case the headless instance option is set propagates
        # it to the Firefox options object
        if self.headless:
            options.headless = True

        # returns the options to the calling method as expected
        # by the current infrastructure
        return options

    def _selenium_capabilities(self, browser):
        return getattr(self, "_selenium_capabilities_%s" % browser)()

    def _selenium_capabilities_chrome(self):
        from selenium.webdriver.common.desired_capabilities import DesiredCapabilities

        capabilities = DesiredCapabilities.CHROME
        capabilities["loggingPrefs"] = dict(browser = "ALL")

        return capabilities

    def _selenium_capabilities_firefox(self):
        from selenium.webdriver.common.desired_capabilities import DesiredCapabilities

        capabilities = DesiredCapabilities.FIREFOX
        capabilities["loggingPrefs"] = dict(browser = "ALL")

        return capabilities

    def _selenium_profile(self, browser):
        return getattr(self, "_selenium_profile_%s" % browser)()

    def _selenium_profile_firefox(self):
        from selenium.webdriver.firefox.firefox_profile import FirefoxProfile

        profile = FirefoxProfile()
        profile.set_preference("browser.cache.disk.enable", False)
        profile.set_preference("browser.cache.memory.enable", False)
        profile.set_preference("browser.cache.offline.enable", False)

        return profile

    def _wait(self, timeout = None):
        from selenium.webdriver.support.ui import WebDriverWait
        kwargs = dict()
        if self.poll_frequency: kwargs["poll_frequency"] = self.poll_frequency
        if timeout == None: timeout = self.owner.timeout
        return WebDriverWait(self.instance, timeout, **kwargs)

    def _move_to(self, element, pivot = "center"):
        element_width = element.size["width"]
        element_height = element.size["height"]

        if pivot == "center":
            return self._move_to_offset(
                element,
                x = int(element_width / 2),
                y = int(element_height / 2)
            )

        if pivot == "left-top":
            return self._move_to_offset(
                element,
                x = 0,
                y = 0
            )

        if pivot == "right-top":
            return self._move_to_offset(
                element,
                x = element_width - 1,
                y = 0
            )

        if pivot == "right-bottom":
            return self._move_to_offset(
                element,
                x = element_width - 1,
                y = element_height - 1
            )

        if pivot == "left-bottom":
            return self._move_to_offset(
                element,
                x = 0,
                y = element_height - 1
            )

    def _move_to_offset(self, element, x = 0, y = 0):
        from selenium.webdriver.common.action_chains import ActionChains

        # creates an action chain that will move the mouse into the
        # target offset of coordinates relative to the element
        actions = ActionChains(self.instance)
        actions.move_to_element_with_offset(element, x, y)
        actions.perform()

        # returns the element that currently has the mouse pointing
        # to it according to the defined offset (if possible)
        return element

    def _move_outside(self, element, raise_e = True):
        """
        Moves the mouse outside a given element, this should
        ensure proper outside of element position (not hover).

        This method is going to use a series of strategies to
        try to achieve the goal of moving the cursor outside
        of the referred element.

        :type element: Element
        :param element: The element to move outside from.
        :type raise_e: bool
        :param raise_e: If an exception should be raised when
        it's not possible to move outside the element.
        """

        from selenium.common.exceptions import MoveTargetOutOfBoundsException

        # gathers the element's dimensions to be able to calculate other
        # corner's offset positions, for the list of possibilities
        size = element.size
        width, height = size["width"], size["height"]

        # creates the tuple that contains the complete set of strategies
        # for the offset operation to move the cursor outside of the
        # requested element (avoiding collision)
        possibilities = (
            (-1, -1),
            (-1, 0),
            (0, -1),
            (width, -1),
            (width, 0),
            (-1, height),
            (0, height),
            (width, height)
        )

        # iterates over the complete set of offset possibilities
        # and runs the cursor to the requested offset, in case
        # there's no exception the control flow is returned (success)
        for x, y in possibilities:
            try:
                self._move_to_offset(element, x, y)
                return
            except MoveTargetOutOfBoundsException:
                pass

        # raises an operation error as it was not possible to move
        # the cursor outside of the current element using any of
        # the available strategies (only active if the raise~
        # exception flag is set as a parameter)
        if raise_e:
            raise appier.OperationalError(
                message = "Couldn't move outside element"
            )

    def _try_visible(self, element, strategy = "scroll_to"):
        from selenium.common.exceptions import MoveTargetOutOfBoundsException

        # prints some debug information on the retry of the visibility
        # test for the element in question
        self.owner.breadcrumbs.debug("Trying visibility on element '%s'" % element.id)

        try:
            # tries to run the cursor movement to the element, this may fail
            # as the element is not currently visible in the viewport
            self._move_to(element)
        except MoveTargetOutOfBoundsException:
            # some drivers raise exceptions when trying to move to elements
            # outside the viewport, we must ignore such exceptions
            pass

        # runs the is visible operation on the element that should guarantee
        # that the element is visible and interactable, if the returns is
        # positive the control flow is returned and the return value is positive
        if self.__is_visible(element): return True

        # executes the try-out strategy to try to make the element visible
        # in the view-port, by default this scrolls the element to the center
        # of the current screen (best possible strategy)
        if strategy == "scroll_to": self.scroll_to(element)

        # moves the element back to the outside of it and so that there's
        # a mouse movement one more time (skeptical move)
        self._move_outside(element, raise_e = False)

        # "resets" the value of the entered flag back to the original value
        # so that the test may be done against the movement to be performed
        self.instance.execute_script("window._entered = false")

        try:
            # tries to run the cursor movement to the element, this may or
            # may not fail according to the success of the strategy
            self._move_to(element)
        except MoveTargetOutOfBoundsException:
            # some drivers raise exceptions when trying to move to elements
            # outside the viewport, we must ignore such exceptions
            pass

        # verifies if the element is visible and interactable and if that's
        # the case returns a positive result and the control flow
        if self.__is_visible(element): return True

        # by default the visibility/interactability of the provided element
        # is assumed to be false (not interactable) as no tests have passed
        return False

    def _resolve_key(self, name):
        from selenium.webdriver.common.keys import Keys
        KEYS = dict(
            enter = Keys.ENTER,
            space = Keys.SPACE,
            backspace = Keys.BACKSPACE,
            left = Keys.LEFT,
            right = Keys.RIGHT,
            up = Keys.UP,
            down = Keys.DOWN,
            page_up = Keys.PAGE_UP,
            page_down = Keys.PAGE_DOWN
        )
        return KEYS[name]

    def _flush_log(self, levels = ("INFO", "WARN", "ERROR")):
        from selenium.common.exceptions import WebDriverException
        try: log = self.instance.get_log("browser")
        except WebDriverException: log = []
        for item in log:
            if not item["level"] in levels: continue
            self.owner.breadcrumbs.info(log)

    def __is_visible(self, element):
        """
        Internal method that verifies the visibility/intractability
        of the provided element taking into account a series of
        previously inserted setup operation.

        Should be used with proper care to avoid unwanted behaviour.

        :type element: Element
        :param element: The element to validated for visibility and/or
        intractability (probably for click purposes).
        :rtype: bool
        :return: If the element is currently interactable.
        """

        if not self.secure and self.instance.execute_script("return window._entered"): return True
        if self.instance.execute_script(
            "return Array.from(arguments[0].parentElement.querySelectorAll(\":hover\").values()).includes(arguments[0])",
            element
        ): return True
        return False



#!/usr/bin/python
# -*- coding: utf-8 -*-

import unittest

import ripe_rainbow

class AssertionsPartTest(unittest.TestCase):

    def test_match_url(self):
        test_case = ripe_rainbow.TestCase()
        assertions_part = ripe_rainbow.AssertionsPart(test_case)

        self.assertEqual(assertions_part.match_url("http://www.platforme.com", "http://www.platforme.com"), True)
        self.assertEqual(assertions_part.match_url("http://www.platforme.com", "http://www.platforme.come"), False)
        self.assertEqual(assertions_part.match_url("http://www.platforme.com", "http://www.platforme.co"), False)

        self.assertEqual(assertions_part.match_url("http://www.platforme.com", "http://www.platforme.com?param1=value1"), True)
        self.assertEqual(assertions_part.match_url("http://www.platforme.com?param1=value1", "http://www.platforme.com"), True)

        self.assertEqual(assertions_part.match_url(
            "http://www.platforme.com?param1=value1",
            "http://www.platforme.com",
            params = dict(param1 = "value1")
        ), True)
        self.assertEqual(assertions_part.match_url(
            "http://www.platforme.com?param1=value1&param2=value2",
            "http://www.platforme.com",
            params = dict(param1 = "value1")
        ), True)
        self.assertEqual(assertions_part.match_url(
            "http://www.platforme.com?param1=value1&param2=value2",
            "http://www.platforme.com",
            params = dict(param1 = "value1"),
            strict = True
        ), False)
        self.assertEqual(assertions_part.match_url(
            "http://www.platforme.com?param1=value1&param2=value2",
            "http://www.platforme.com",
            params = dict(param1 = "value1", param2 = "value2"),
            strict = True
        ), True)

        self.assertEqual(assertions_part.match_url(
            "http://www.platforme.com?param1=value1&param2=value2#anchor1",
            "http://www.platforme.com",
            params = dict(param1 = "value1", param2 = "value2"),
            fragment = "anchor1",
            strict = True
        ), True)



#!/usr/bin/python
# -*- coding: utf-8 -*-

def test_case_fullname(test_case, use_ctx = True):
    use_ctx = use_ctx and hasattr(test_case, "ctx")
    ctx = test_case.ctx if use_ctx else None
    ctx_s = "%s." % ctx if ctx else ""
    module = test_case.__class__.__module__
    if module == None or module == str.__class__.__module__:
        return ctx_s + test_case.__class__.__name__
    return ctx_s + module + "." + test_case.__class__.__name__

def test_fullname(test, use_ctx = False):
    instance = test.__self__
    return test_case_fullname(instance, use_ctx = use_ctx) + "." + test.__name__



#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

from .. import parts

class InteractionsPart(parts.Part):

    def get_url(self, url, params = [], fragment = ""):
        """
        Navigates to a certain URL with given GET parameters and to
        the request fragment.

        The operation is always going to be performed using a GET
        request as that's the only available request method from
        the browser.

        :type url: String
        :param url: The URL to navigate to, this should represent only
        the base URL without GET parameters and fragment.
        :type params: list
        :param params: A list of (key, values) tuples representing
        the GET query parameters to be added to the base URL
        :type fragment: String
        :param fragment: The fragment string to be added to the last part
        of the URL to be built.
        :rtype: bool
        :return: The result of the redirection, if it was verified by
        the browser.
        """

        params_s = []

        for (key, value) in params:
            key_q = appier.util.quote(key)
            for _value in value:
                value_q = appier.util.quote(_value)
                param = key_q + "=" + value_q
                params_s.append(param)

        params_s = "&".join(params_s) if params_s else ""

        if params_s: url += "?" + params_s
        if fragment: url += "#" + fragment

        self.driver.get(url)

        return self.waits.redirected_to(url)

    def write_text(self, selector, text):
        """
        Writes the text in the given element, this means having
        it typed like in a physical keyboard.

        :type selector: String
        :param selector: The selector for the element to write the text in.
        :type text: String
        :param text: The text to write "using the keyboard".
        :rtype: Element
        :return: The element with the text changed.
        """

        # waits for the element to be available at the DOM
        element = self.waits.element(selector)

        # waits until a valid text change in the element is possible, this
        # overcomes limitations with non interactable elements
        return self.waits.until(
            lambda d: self.driver.safe(self.driver.write_text, element, text),
            "Element '%s' found but never became writable" % selector
        )

    def press_key(self, selector, key):
        """
        Presses the provided key on a certain element, pressed like having
        the proper enter key pressed.

        :type selector: String
        :param selector: The selector for the element to focus when
        pressing enter.
        :type key: String
        :param key: The name of the key that is going to be pressed by
        the keyboard, this name is set on an agnostic way.
        :rtype: Element
        :return: The element with the key pressed.
        """

        # waits for the element to be available at the DOM
        element = self.waits.element(selector)

        # waits until a valid key stroke in the element is possible, this
        # overcomes limitations with non interactable elements
        return self.waits.until(
            lambda d: self.driver.safe(self.driver.press_key, element, key),
            "Element '%s' found but never became interactable" % selector
        )

    def press_enter(self, selector):
        """
        Presses the enter key on a certain element, pressed like having
        the proper enter key pressed.

        :type selector: String
        :param selector: The selector for the element to focus when
        pressing enter.
        :rtype: Element
        :return: The element with the enter key pressed.
        """

        # waits for the element to be available at the DOM
        element = self.waits.element(selector)

        # waits until a valid key stroke in the element is possible, this
        # overcomes limitations with non interactable elements
        return self.waits.until(
            lambda d: self.driver.safe(self.driver.press_enter, element),
            "Element '%s' found but never became interactable" % selector
        )

    def click_when_possible(self, selector, condition = None):
        """
        Clicks an element when possible, which happens when that element is both
        visible and "clickable".

        :type selector: String
        :param selector: The selector for the element to click.
        :type condition: Function
        :param condition: The filter the selected element must pass to be "clickable".
        :rtype: Element
        :return: The clicked element if there's any otherwise an invalid value.
        """

        # waits for the element to be available at the DOM
        element = self.waits.element(selector, condition = condition)

        # waits until the try click operation is possible meaning that a
        # proper click has been "done" by the driver
        return self.waits.until(
            lambda d: self.driver.safe(self.driver.click, element),
            "Element '%s' found but never became clickable" % selector
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

from .. import parts

class WaitsPart(parts.Part):

    def until(self, method, message = None, timeout = None):
        return self.driver.wrap_outer(
            lambda: self.driver._wait(timeout = timeout).until(
                lambda *args: self.driver.wrap_inner(method, *args),
                message = message
            )
        )

    def redirected_to(
        self,
        url,
        params = None,
        fragment = None,
        starts_with = False,
        timeout = None
    ):
        return self.until(
            lambda d: self.assertions.at_url(
                url,
                params = params,
                fragment = fragment,
                starts_with = starts_with
            ),
            message = "Expecting the page to be '%s' but is '%s'" % (
                url,
                self.driver.current_url
            ),
            timeout = timeout
        )

    def all(self, selector, condition, timeout = None):
        return self.until(
            lambda d: self.assertions.all(selector, condition),
            message = "Elements for '%s' don't match the condition" % selector,
            timeout = timeout
        )

    def element(self, selector, condition = None, timeout = None):
        return self.until(
            lambda d: self.assertions.exists(selector, condition = condition),
            message = "Element '%s' not found" % selector,
            timeout = timeout
        )

    def elements(self, selector, condition = None, timeout = None):
        return self.until(
            lambda d: self.assertions.exists_multiple(selector, condition = condition),
            message = "Elements '%s' not found" % selector,
            timeout = timeout
        )

    def text(self, selector, text, timeout = None, ensure = False):
        return self.until(
            lambda d: self.assertions.has_text(selector, text, ensure = ensure),
            message = "Element '%s' with text '%s' not found" % (selector, text),
            timeout = timeout
        )

    def is_visible(self, selector, condition = None, timeout = None):
        return self.until(
            lambda d: self.assertions.is_visible(selector, condition = condition),
            message = "Element '%s' is not visible" % selector,
            timeout = timeout
        )

    def is_not_visible(self, selector, condition = None, timeout = None):
        return self.until(
            lambda d: self.assertions.is_not_visible(selector, condition = condition),
            message = "Element '%s' is visible" % selector,
            timeout = timeout
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

import unittest

import ripe_rainbow

from . import mock

class UtilTest(unittest.TestCase):

    def test_test_fullname(self):
        test_case = mock.DemoTestCase()
        result = ripe_rainbow.test_fullname(test_case.test_empty)

        self.assertEqual(result, "ripe_rainbow.unit.mock.DemoTestCase.test_empty")



#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

class SkipError(appier.AppierException):

    def __init__(self, *args, **kwargs):
        appier.AppierException.__init__(self, *args)
        self.reason = kwargs.get("reason", None)

class TimeoutError(appier.AppierException):
    pass



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import setuptools

setuptools.setup(
    name = "ripe-rainbow",
    version = "0.5.14",
    author = "Platforme International",
    author_email = "development@platforme.com",
    description = "RIPE Rainbow",
    license = "Apache License, Version 2.0",
    keywords = "ripe rainbow test",
    url = "http://www.platforme.com",
    zip_safe = False,
    packages = [
        "ripe_rainbow",
        "ripe_rainbow.domain",
        "ripe_rainbow.domain.base",
        "ripe_rainbow.domain.logic",
        "ripe_rainbow.interactive",
        "ripe_rainbow.unit",
        "ripe_rainbow.unit.domain",
        "ripe_rainbow.unit.domain.base"
    ],
    test_suite = "ripe_rainbow.unit",
    package_dir = {
        "" : os.path.normpath("src")
    },
    entry_points = {
        "console_scripts" : [
            "rainbow = ripe_rainbow.main:main",
            "ripe-rainbow = ripe_rainbow.main:main"
        ]
    },
    install_requires = [
        "appier",
        "appier_console",
        "appier_extras_api"
    ],
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Topic :: Utilities",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7"
    ]
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from . import domain
from . import interactive
from . import decorators
from . import errors
from . import info
from . import loaders
from . import results
from . import runners
from . import test_cases
from . import util

from .domain import *
from .interactive import *
from .decorators import test
from .errors import SkipError
from .info import NAME, VERSION, RAINBOW, LABEL
from .loaders import Loader, PathLoader
from .results import Result
from .runners import Runner, ConsoleRunner
from .test_cases import TestCase
from .util import test_case_fullname, test_fullname
